﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml;
using System.Xml.XPath;

namespace tp.chap._2.Classes
{
    public static class Utilities
    {
        static string fileName;
        static XmlTextReader reader;
        static XmlDocument doc;
        static XPathNavigator nav;
        static XPathExpression expr;
        static XPathNodeIterator iterator;

        public static void LoadXmlDocument()
        {
            try
            {
                fileName =  "..\\..\\Data\\catalog.xml";
                doc = new XmlDocument();
                nav = doc.CreateNavigator();
                reader = new XmlTextReader(fileName);
                doc.Load(reader);
                reader.Close();
            }
            catch (Exception ex)
            {
                Debug.WriteLine(ex.Message);
            }
        }

        public static void PrintAllElements()
        {
            try
            {
                expr = nav.Compile("/catalog/cd");
                iterator = nav.Select(expr);

                while (iterator.MoveNext())
                {
                    XPathNavigator nav2 = iterator.Current.Clone();
                    Debug.WriteLine("ID: " + nav2.GetAttribute("id", "") + "\n"
                    + "country: " + nav2.SelectSingleNode("country") + "\n"
                    + "title: " + nav2.SelectSingleNode("title") + "\n"
                    + "artist: " + nav2.SelectSingleNode("artist") + "\n"
                    + "price: " + nav2.SelectSingleNode("price")
                );
                }
            }
            catch (Exception ex)
            {
                Debug.WriteLine(ex.Message);
            }
        }

        public static void PrintElementById(int id)
        {
            try
            {
                expr = nav.Compile("/catalog/cd[@id='" + id.ToString() + "']");
                iterator = nav.Select(expr);
                iterator.MoveNext();
                XPathNavigator nav2 = iterator.Current.Clone();
                Debug.WriteLine("ID: " + nav2.GetAttribute("id", "") + "\n"
                + "country: " + nav2.SelectSingleNode("country") + "\n"
                + "title: " + nav2.SelectSingleNode("title") + "\n"
                + "artist: " + nav2.SelectSingleNode("artist") + "\n"
                + "price: " + nav2.SelectSingleNode("price"));
            }
            catch (Exception ex)
            {
                Debug.WriteLine(ex.Message);
            }
        }

        public static void PrintLastElementPrice()
        {
            try
            {
                expr = nav.Compile("//cd[last()]");
                iterator = nav.Select(expr);
                iterator.MoveNext();
                XPathNavigator nav2 = iterator.Current.Clone();
                Debug.WriteLine("price of last element: " + nav2.SelectSingleNode("price"));
            }
            catch (Exception ex)
            {
                Debug.WriteLine(ex.Message);
            }
        }

        public static void XpathFunctions()
        {
            try
            {
                Debug.WriteLine(nav.Evaluate("count(/catalog/cd)").ToString());
                Debug.WriteLine(nav.Evaluate("count(/catalog/cd[country='UK'])").ToString());
                Debug.WriteLine(nav.Evaluate("count(/catalog/cd[price>'10' and country = 'USA'])").ToString());

                Debug.WriteLine(nav.Evaluate("sum(/catalog/cd[price>='10']/price)").ToString());
            }
            catch (Exception ex)
            {
                Debug.WriteLine(ex.Message);
            }
        }

        public static void ReplaceElementById(int id, string title, string artist, double price)
        {
            try
            {
                expr = nav.Compile("/catalog/cd[@id='" + id.ToString() + "']");
                iterator = nav.Select(expr);
                iterator.MoveNext();
                XPathNavigator nav2 = iterator.Current.Clone();
                nav2.InnerXml = "<title>" + title + "</title>" +
                            "<artist>" + artist + "</artist>" +
                            "<price>" + price.ToString() + "</price>";

                doc.Save(fileName);
            }
            catch (Exception ex)
            {
                Debug.WriteLine(ex.Message);
            }
        }

        public static void AppendElement(string title, string artist, double price)
        {
            try
            {
                int maxID = Convert.ToInt32(nav.Evaluate("count(/catalog/cd)")) + 1;
                nav.MoveToChild("catalog", "");
                XmlReader newCd = XmlReader.Create(new StringReader(
                                            "<cd id='" + maxID.ToString() + "'>" +
                                            "<title>" + title + "</title>" +
                                            "<artist>" + artist + "</artist>" +
                                            "<price>" + price.ToString() + "</price>"
                                            +"</cd>")
                                         );

                nav.AppendChild(newCd);
                doc.Save(fileName);
            }
            catch (Exception ex)
            {
                Debug.WriteLine(ex.Message);
            }
        }

        public static void InsertAfterId(int id, string title, string artist, double price)
        {
            try
            {
                int maxID = Convert.ToInt32(nav.Evaluate("count(/catalog/cd)")) + 1;

                expr = nav.Compile("/catalog/cd[@id='" + id.ToString() + "']");
                iterator = nav.Select(expr);
                iterator.MoveNext();
                XPathNavigator nav2 = iterator.Current.Clone();
                
                XmlReader newCd = XmlReader.Create(new StringReader(
                                            "<cd id='" + maxID.ToString() + "'>" +
                                            "<title>" + title + "</title>" +
                                            "<artist>" + artist + "</artist>" +
                                            "<price>" + price.ToString() + "</price>"
                                            + "</cd>")
                                         );

                nav2.InsertAfter(newCd);
                doc.Save(fileName);
            }
            catch (Exception ex)
            {
                Debug.WriteLine(ex.Message);
            }
        }

        public static void AddTaxToPrices (double tax)
        {
            try
            {
                double val, taxedVal;
                foreach (XPathNavigator n in nav.Select("//price"))
                {
                    val = Convert.ToDouble(n.Value);
                    taxedVal = val * (1+tax);
                    n.SetValue(taxedVal.ToString());
                }
                doc.Save(fileName);
                PrintAllElements();
            }
            catch (Exception ex)
            {
                Debug.WriteLine(ex.Message);
            }
        }
    }
}
#region unused
//XmlElement root = doc.DocumentElement;
//XmlNode oldCd = root.SelectSingleNode("/catalog/cd[@id='" + id.ToString() + "']");

//XmlElement newCd = doc.CreateElement("cd");
//newCd.SetAttribute("id", id.ToString());

//newCd.InnerXml = "<title>" + title + "</title>" +
//        "<artist>" + artist + "</artist>" +
//        "<price>" + price.ToString() + "</price>";
//////////////////////////////////////////////////////////////////////////////////////////////
//XmlElement root = doc.DocumentElement;
//int maxID = Convert.ToInt32(nav.Evaluate("count(/catalog/cd)")) + 1;

//XmlElement newCd = doc.CreateElement("cd");
//newCd.SetAttribute("id", maxID.ToString());

//newCd.InnerXml = "<title>" + title + "</title>" +
//        "<artist>" + artist + "</artist>" +
//        "<price>" + price.ToString() + "</price>";

//root.AppendChild(newCd);
#endregion